#!/usr/bin/env python
##------------------------------------------------------------------------------
##
## Name: fix_acs_wfc_DGX.py
##
##  Pre-SM4 gain values for ACS/WFC are  1,   2,   4,   and 8
##  as opposed to the post-SM4 values of 0.5, 1.0, 1.4, and 1.0
##  Front end prefered integers, therefore multiplied post-SM4 by 10
##  giving post-SM4 DGX values of        5,   10,  14,  and 20
##  We want the real values, not ones increased by a factor of 10
##
##  This script is called at the end of SSKEY after the DGX file
##  has been created.  It should only be called for ACS WFC observations.
##  It will divide the post-SM4 DGX values by 10 and  REWRITE the DGX file.
##
## Usage: 
##          fix_acs_wfc_DGX.py <dgx filename>
## 
## Returns:
##          OK     (0)
##          NOT_OK (1)
## 
##
## mm/dd/yy PR    who        Description
## -------- ----- ---------- ---------------------------------------------------
## 09/07/08 60640 Sherbert   modify SS_GAIN for post-SM4 ACS/WFC
##
##------------------------------------------------------------------------------

import sys
import os
import string
import re
import opusutil

moduleName = os.path.basename(sys.argv[0])
## exit codes
OK     =  0             ## NOT same as DV_OK, but same as SYS_SUCCESS
NOT_OK =  1             ## UNIX error indicator

def PrintUsage():
    line1 = "Usage: "   + moduleName + " <dgx_filename> \n"
    line2 = "Example: " + moduleName + " jwfc1104q.dgx \n"
    usageMsg = "\n" + line1 + line2
    print usageMsg
    return

def cleanUp (thisFile):
    ## Clean up when not writing temporary file
    msg = "Clean up " + thisFile
    opusutil.PrintMsg("D", msg, moduleName)
    try:
        os.remove(thisFile)
    except:
        msg = "ERROR removing " + thisFile
        opusutil.PrintMsg("E", msg, moduleName)
    return

def fix_acs_wfc_DGX():
    if len(sys.argv) != 2:
        PrintUsage()
        sys.exit(NOT_OK)
    dgxFile = sys.argv[1]
    
    ## Create a Template for modifying the SS_GAIN line in the dgx file
    ss_gainTemplate = string.Template("SS_GAIN = $new_gain $spaces / ACS JFI    0")
    ## For a 3-digit floating point number, 
    ## we need 16 spaces from space AFTER integer to space BEFORE slash 
    spaces = ' ' * 16
    
    
    #lnum = 0                                ## for curiosity
    check_gain = 0
    dgxFilObj = open( dgxFile, 'r' )
    dgxLines  = dgxFilObj.readlines()
    dgxFilObj.close()
    
    tmpFile = dgxFile + '_tmp'
    tmpFilObj = open( tmpFile, 'w' )
    for line in dgxLines :
    #   lnum+=1
        if line[0:6] == "CONFIG":
            bits = line.split()
            if "ACS/WFC" == bits[2] :
                msg = "Found an ACS/WFC observation. "
                opusutil.PrintMsg("I", msg, moduleName)
                check_gain = 1
        if line[0:7] == "SS_GAIN" and check_gain:
                bits = line.split()
               #print "bits of SS_GAIN line are: ", bits
                try:
                    gain = int(bits[2])
                except ValueError:
                    msg = "SS_GAIN value not an integer: " + \
                          str(bits[2]) + ".  No change made."
                    opusutil.PrintMsg("I", msg, moduleName)
                    tmpFilObj.close()
                    cleanUp(tmpFile)
                    sys.exit(OK)
               #print "gain     is ", gain
                if  5 == gain or 10 == gain or 14 == gain or 20 == gain:
                    msg = "Calculate new_gain: " + str(gain) + " / 10."
                    opusutil.PrintMsg("I", msg, moduleName)
    
                    new_gain = gain / 10.
    
                    msg = "new_gain is " + str(new_gain)
                    opusutil.PrintMsg("I", msg, moduleName)
                    ## Write modified line to temp file
                    line = ss_gainTemplate.substitute(locals()) + "\n"
                else:
                    msg = str(gain) + \
                          " is not a post-SM4 SS_GAIN value. " + \
                          " No change necessary."
                    opusutil.PrintMsg("I", msg, moduleName)
                    tmpFilObj.close()
                    cleanUp(tmpFile)
                    sys.exit(OK)
        tmpFilObj.write(line)
    
    tmpFilObj.close()
    ## All the changes are tmpFile, so overwrite dgxFile with it
    try:
        os.rename(tmpFile,dgxFile)
    except OSError, (errno, errstr):
        msg = "Rename failed, DGX NOT corrected? " + errstr 
        opusutil.PrintMsg("E", msg, moduleName)
        sys.exit(NOT_OK)
    
    ## This returns 0 (No change) 
    return OK


##------------------------------------------------------------------------------
## The main routine
##------------------------------------------------------------------------------
status = fix_acs_wfc_DGX()
sys.exit(status)
